@@ -31,7 +31,7 @@ function lifecycle (pkg, stage, wd, unsafe, failOk, cb) {
   if (!pkg) return cb(new Error("Invalid package data"))
 
   log.info(stage, pkg._id)
-  if (!pkg.scripts) pkg.scripts = {}
+  if (!pkg.scripts || npm.config.get('ignore-scripts')) pkg.scripts = {}
 
   validWd(wd || path.resolve(npm.dir, pkg.name), function (er, wd) {
     if (er) return cb(er)
@@ -136,14 +136,32 @@ function runPackageLifecycle (pkg, env, wd, unsafe, cb) {
 
   var note = "\n> " + pkg._id + " " + stage + " " + wd
            + "\n> " + cmd + "\n"
-  console.log(note)
-  runCmd(cmd, pkg, env, stage, wd, unsafe, cb)
+  runCmd(note, cmd, pkg, env, stage, wd, unsafe, cb)
 }
 
-function runCmd (cmd, pkg, env, stage, wd, unsafe, cb) {
+
+var running = false
+var queue = []
+function dequeue() {
+  running = false
+  if (queue.length) {
+    var r = queue.shift()
+    runCmd.apply(null, r)
+  }
+}
+
+function runCmd (note, cmd, pkg, env, stage, wd, unsafe, cb) {
+  if (running) {
+    queue.push([note, cmd, pkg, env, stage, wd, unsafe, cb])
+    return
+  }
+
+  running = true
+  log.pause()
   var user = unsafe ? null : npm.config.get("user")
     , group = unsafe ? null : npm.config.get("group")
 
+  console.log(note)
   log.verbose("unsafe-perm in lifecycle", unsafe)
 
   if (process.platform === "win32") {
@@ -159,13 +177,12 @@ function runCmd (cmd, pkg, env, stage, wd, unsafe, cb) {
   }
 }
 
-function runCmd_ (cmd, pkg, env, wd, stage, unsafe, uid, gid, cb) {
-  var sh = "sh"
-  var shFlag = "-c"
+function runCmd_ (cmd, pkg, env, wd, stage, unsafe, uid, gid, cb_) {
 
-  if (process.platform === "win32") {
-    sh = "cmd"
-    shFlag = "/c"
+  function cb (er) {
+    cb_.apply(null, arguments)
+    log.resume()
+    process.nextTick(dequeue)
   }
 
   var conf = { cwd: wd
@@ -174,13 +191,24 @@ function runCmd_ (cmd, pkg, env, wd, stage, unsafe, uid, gid, cb) {
              }
 
   if (!unsafe) {
-    conf.uid = uid
-    conf.gid = gid
+    conf.uid = uid ^ 0
+    conf.gid = gid ^ 0
+  }
+
+  var sh = "sh"
+  var shFlag = "-c"
+
+  if (process.platform === "win32") {
+    sh = "cmd"
+    shFlag = "/c"
+    conf.windowsVerbatimArguments = true
   }
 
   var proc = spawn(sh, [shFlag, cmd], conf)
-  proc.on("close", function (code) {
-    if (code) {
+  proc.on("close", function (code, signal) {
+    if (signal) {
+      process.kill(process.pid, signal);
+    } else if (code) {
       var er = new Error("Exit status " + code)
     }
     if (er && !npm.ROLLBACK) {
@@ -216,7 +244,9 @@ function runHookLifecycle (pkg, env, wd, unsafe, cb) {
 
   fs.stat(hook, function (er) {
     if (er) return cb()
-    runCmd(hook, pkg, env, stage, wd, unsafe, cb)
+    var note = "\n> " + pkg._id + " " + stage + " " + wd
+             + "\n> " + cmd
+    runCmd(note, hook, pkg, env, stage, wd, unsafe, cb)
   })
 }
 
